require 'util'

local default_lists = {
  ['yellow-belt'] = {'transport-belt','underground-belt','splitter'},
  ['red-belt'] = {'fast-transport-belt','fast-underground-belt','fast-splitter'},
  ['blue-belt'] = {'express-transport-belt','express-underground-belt','express-splitter'},
  ['poles'] = {'small-electric-pole','medium-electric-pole','big-electric-pole','substation'},
  ['pipes'] = {'pipe','pipe-to-ground','pump','storage-tank'},
  ['trains'] = {'locomotive','cargo-wagon','fluid-wagon','artillery-wagon'},
  ['flooring'] = {'stone-brick','concrete','hazard-concrete','refined-concrete','refined-hazard-concrete','landfill'},
  ['solar'] = {'solar-panel','accumulator'},
  ['heat'] = {'heat-pipe','heat-exchanger'},
  ['signals'] = {'rail-signal','rail-chain-signal'},
  ['inserters'] = {'long-handed-inserter','fast-inserter','stack-inserter'},
  
  }

local function update_links (pindex)
  local lists = global.pdata[pindex].lists
  local links = {}
  for _,list in pairs(lists) do
    if #list ~= 1 then --lists of length one don't need cycling
      for i=1,#list do
        if i==1 then --first list item
          links[list[i]] = {next = list[i+1], prev = list[#list], len = #list}
        elseif i==#list then --last list item
          links[list[i]] = {next = list[  1], prev = list[ i-1 ], len = #list}
        else --all other list items
          links[list[i]] = {next = list[i+1], prev = list[ i-1 ], len = #list}
          end
        end
      end
    end
  global.pdata[pindex].links = links
  -- print(serpent.block(links))
  end

script.on_event(defines.events.on_player_created,function(e)
  local pindex = e.player_index
  global.pdata = global.pdata or {}
  global.pdata[pindex] = global.pdata[pindex] or {}
  local pdata = global.pdata[pindex]
  pdata.lists = pdata.lists or util.table.deepcopy(default_lists)
  update_links(pindex)
  end)
  
script.on_event({'er-stack-swap-prev','er-stack-swap-next'},function(e)
  local p = game.players[e.player_index]
  if p.selected then return end --don't interfere with normal entity rotation
  if not (p.cursor_stack.valid and p.cursor_stack.valid_for_read) then return end --empty cursor
  local links = global.pdata[p.index].links
  local cursor = p.cursor_stack
  if not links[cursor.name] then return end --not a linked item
  -- determine target item
  local mode = (e.input_name == 'er-stack-swap-next') and 'next' or 'prev'
  local target_name = links[cursor.name][mode]
  local target_stack
  -- find a valid stack, skip to next target if there is none
  for i=1,links[cursor.name].len-1 do
    target_stack = p.get_inventory(defines.inventory.player_quickbar).find_item_stack(target_name)
    if not target_stack then
      target_stack = p.get_inventory(defines.inventory.player_main).find_item_stack(target_name)
      end
    if target_stack then break end --found! no need to search further
    target_name = links[target_name][mode]
    end
  --have we found a stack? can it be put to the cursor?
  if (not target_stack) or (not p.clean_cursor()) then return end
  --put item to cursor
  p.cursor_stack.transfer_stack(target_stack)

  end)
  

  
commands.add_command('stackrotate','list/add/remove',function(e)
  local p = game.players[e.player_index]
  -- print(serpent.block(e))
  --LIST
  if string.sub(e.parameter,1,4) == 'list' then
    local lists = global.pdata[p.index].lists
    for k,v in pairs(lists) do
      p.print(serpent.line({[k]=v}))
      end
  --ADD
  elseif string.sub(e.parameter,1,3) == 'add' then
    local tbl = load('return '..string.sub(e.parameter,4))
    tbl = (type(tbl) == 'function') and tbl()
    if type(tbl) == 'table' then
      for key,value in pairs(tbl) do
        global.pdata[p.index].lists[key] = value
        end
    else
      p.print('Not a table.')
      end
  --REMOVE
  elseif string.sub(e.parameter,1,6) == 'remove' then
    global.pdata[p.index].lists[string.sub(e.parameter,8)] = nil
  --TELL
  elseif string.sub(e.parameter,1,4) == 'tell' then
    if p.cursor_stack.valid_for_read then
      p.print(p.cursor_stack.name)
      end
  --RESET
  elseif string.sub(e.parameter,1,5) == 'reset' then
    global.pdata[p.index].lists = util.table.deepcopy(default_lists)
  --INVALID
  else
    p.print('Unrecognized stackrotate command.')
    end
  update_links(p.index)
  end)


  
  
  
  
  